SystemOrganization addCategory: #Sensors!

TestCase subclass: #DeviceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!DeviceTest methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 10:05'!
testXml
	|xml dev stream xmlString doc device|
	xmlString := '<Device><name>arduino</name><ip>10.9.13.11</ip></Device>'.
	stream := xmlString readStream.
	doc := XMLDOMParser parseDocumentFrom: stream.
	device := Device new.
	device name: (doc findElementNamed: 'name') contentString.
	device ip: (doc findElementNamed: 'ip') contentString.
	
	Transcript show: device asXml.! !

TestCase subclass: #ValueWithTimeStampTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!ValueWithTimeStampTest methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:53'!
test
	| history t1 t2 t3 t4 |
	t1 := DateAndTime yesterday.
	t2 := DateAndTime today.
	t3 := DateAndTime now.
	t4 := DateAndTime tomorrow.
	(history := ValueWithTimeStamp new) at: t1 become: 1.
	history at: t4 become: 4.
	history at: t3 become: 3.
	history at: t2 become: 2.
	
	self assert: (history valueAt: t1) = 1.
	self assert: (history valueAt: t2) = 2.
	self assert: (history valueAt: t3) = 3.
	self assert: (history valueAt: t4) = 4.
	
	Transcript show: history.
	Transcript show: (history timeStampAndValue).
	Transcript show: 'testEnd'; cr.! !

WARestfulHandler subclass: #DeviceWS
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 22:56'!
addDevice
	|xml device|
    <post>
    <path: 'Device'>
	xml := (self requestContext request) rawBody.
	Transcript show: xml.
	device := Device newFromXml: xml.
	DeviceManager singleton addDevice: device.
	^'<Device>', (device name) , '</Device>'.! !

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 21:25'!
getDevice: deviceName
    <get>
    <path: 'Device/{1}'>
    <produces: 'text/xml'>
	^(DeviceManager singleton getDevice: deviceName) asXml.! !

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 21:25'!
getDevices
    <get>
    <path: 'Device'>
    <produces: 'text/xml'>
	|stream|
	stream := WriteStream on: (String new: 1).
	stream nextPutAll: '<Devices>'.
	DeviceManager singleton devices valuesDo: [:device | 
			stream nextPutAll: device asXml.
		].
	stream nextPutAll: '</Devices>'.
	^stream contents.! !

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 22:45'!
postData: aDeviceName sensorType: aSensorType 
	|xml device doc stream data sensor|
    <post>
    <path: 'Data/{1}/{2}'>
	xml := (self requestContext request) rawBody.
	Transcript show: xml.
	stream := xml readStream.
	doc := XMLDOMParser parseDocumentFrom: stream.
	data := (doc findElementNamed: 'Data') contentString asNumber.
	
	sensor := (DeviceManager getDevice: aDeviceName) getSensorByType: aSensorType.
	
	sensor updateDataNow: data.
	
	^'Received <Data>', data asString ,'</Data> for: ', aDeviceName, ' ', aSensorType.! !

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 21:25'!
removeDevice: deviceName
    <delete>
    <path: 'Device/{1}'>
    <produces: 'text/xml'>
   	DeviceManager singleton removeDevice: deviceName.
	^'<Device>', deviceName , '</Device>'.! !

SortedCollection variableSubclass: #SortedSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!SortedSequence commentStamp: '<historical>' prior: 0!
I add one method to SortedCollection.  It would be just as easy to add the method
directly to the class instead of making a subclass, but this makes it easier to see
the new method.

The method is indexOfStartOfIntervalContaining: anElement
It returns the index of anElement or, if it is not present, the index of the
largest element smaller than it.  It is useful when you want to have a
table that is sorted by keys and you want to look up in the table to
find the nearest match to a key.!

!SortedSequence methodsFor: 'accessing' stamp: 'WeiqiLuo 7/6/2013 10:52'!
indexOfStartOfIntervalContaining: anElement 
	"Return the index of anElement or, if it is not present, of the index of the
	largest element smaller than it."

	self isEmpty ifTrue: [^0].
	^(self indexForInserting: anElement) - firstIndex! !

WAComponent subclass: #WebAlarmView
	instanceVariableNames: 'threshold name device alarmName deviceName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 15:29'!
renderContentOn: html
	| alarm |
	html
		form: [ 
			html text: 'Alarm Name'.
			html textInput callback: [ :value | alarmName := value ].
			html break.
			html text: 'Device'.
			html textInput callback: [ :value | deviceName := value ].
			html break.
			html text: 'Threshold'.
			html textInput callback: [ :value | threshold := value ].
			html break.
			html submitButton on: #cancel of: self.
			html submitButton
				callback: [ 
							alarm := Alarm new setAlarm: alarmName device: deviceName threshold: threshold.
							DeviceManager singleton addAlarm: alarm.
							^ self answer ];
				with: 'Save' ]! !

WAComponent subclass: #WebComponent
	instanceVariableNames: 'header'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebComponent methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 01:56'!
children

^ Array with: header! !

!WebComponent methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 16:04'!
initialize

super initialize.
header := WebHeader new! !

!WebComponent methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 11:07'!
style
	^ '
		html, body {
			background: #fefefe;
			font-family: verdana;
			color: #3e3e3e;
			font-size:13px;
		}
		
		h1, h2, h3 {
			color: #4f4f4f;
			font-size:20px;
		}
		
		h2 {
			font-size:16px;
		}
		
		.action {
			padding: 0 15px 0 15px;
		}
		
		hr {
			margin 5px 0 5px 0;
		}
		
		.historical span {
			width:100px;
		}
		.historyDetail {
			padding-top:5px;
		}

		.historyDetail+div{
			padding:0 0 20px 0;
		}
	'! !

WebComponent subclass: #WebConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebConfiguration methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 15:28'!
renderContentOn: html
	html render: header.
	html heading: 'Alarms'.
	html unorderedList: [ DeviceManager singleton alarms do: [ :alarm | html listItem: [ html text: alarm name ] ] ].
	html anchor
		callback: [ self call: WebAlarmView new ];
		with: 'Add Alarm'.
	html break.
	self renderNavigationMenu: html! !

!WebConfiguration methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 11:06'!
renderNavigationMenu: html
	html
		anchor
				callback: [ self returnHome ];
				with: 'Return Home' ! !

!WebConfiguration methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 11:06'!
returnHome

	self answer! !

WebComponent subclass: #WebHome
	instanceVariableNames: 'deviceWS'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebHome class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 00:37'!
canBeRoot

^ true! !

!WebHome class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 17:13'!
initialize

	| app |

	app := WAAdmin register: self asApplicationAt: 'sensorHome'.

	app

		addLibrary: JQDevelopmentLibrary;

		addLibrary: HCDevelopmentLibrary.

	app preferenceAt: #scriptGeneratorClass put: JQScriptGenerator.! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 14:54'!
renderContentOn: html

html div
	id: header;
	with: [
	html render: header].
	
	html unorderedList: [
 		html listItem:  'Notifications';
			listItem: 'Lights';
			listItem: 'Temperature';
		 	listItem: 'Reports';
			listItem: 'Configure'].
	
	html anchor
		class: 'action';
		callback: [self call: (WebReports new)];
		with: 'Reports'.
		
	html anchor
		callback: [self call: (WebConfiguration new)];
		with: 'Configuration'.! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 14:55'!
style
	^ '
		html, body {
			background: #fefefe;
			font-family: verdana;
			color: #3e3e3e;
			font-size:13px;
		}
		
		#header {
			background: #000000;
		}
		
		h1, h2, h3 {
			color: #4f4f4f;
			font-size:20px;
		}
		
		h2 {
			font-size:16px;
		}
		
		.action {
			padding: 0 50px 0 15px;
		}
		
		hr {
			margin 5px 0 5px 0;
		}
		
		.historical span {
			width:100px;
		}
		.historyDetail {
			padding-top:5px;
		}

		.historyDetail+div{
			padding:0 0 20px 0;
		}
	'! !

WebComponent subclass: #WebReports
	instanceVariableNames: 'deviceList selectDevice'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 16:21'!
getDevicesForList
	| deviceList deviceNames |
	deviceList := DeviceManager singleton devices.
	deviceNames := OrderedCollection new.
	deviceList valuesDo: [ :value | deviceNames add: value name ].
	^ deviceNames! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 16:41'!
initialize 

super initialize.
! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 17:01'!
renderBarChartOn: html
	html div
		script:
			(html
				highchart: [ :chart | 
					chart chart defaultSeriesType: 'pie'.
					chart title text: 'Light On/Off'.
					chart series: (Array with: (chart step data: #(#('On' 80) #(#Off 20)))) ])! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 00:00'!
renderContentOn: html

	deviceList := self getDevicesForList.
	html render: header.
	html heading: 'Reports'.
	self renderDeviceList: html.
	
	self renderNavigationMenu: html! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 02:06'!
renderDeviceList: html
	|device rawdata xdata ydata chart|
	html
		form: [ 
			html select
				list: deviceList;
				size: 2;
				selected: deviceList first;
				callback: [ :value | selectDevice := value].
				html break.
				html submitButton
         			callback: [device := DeviceManager singleton getDevice: selectDevice.
					rawdata := device data.
					Transcript show: rawdata.
					xdata := OrderedCollection new.
					ydata := OrderedCollection new.
					rawdata do: [:each | xdata add: (each timestamp). ydata add: (each measurement)].
					chart := (WebLineChart new) xData: xdata yData: ydata.
					self call: chart.
					];			
				with: 'Submit' ]! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 17:07'!
renderLineChartOn: html
	html div
		script:
			(html
				highchart: [ :chart | 
					chart chart defaultSeriesType: 'line'.
					chart title text: 'Temperature'.
					chart xAxis categories: #('1pm' '2pm' '3pm').
					chart yAxis title text: 'Degrees Fahrenheit'.
					chart
						series:
							(Array
								with:
									(chart step
										name: 'Reference';
										data: #(65 70 70))
								with:
									(chart step
										name: 'Measured';
										data: #(75 72 72))) ])
									
					! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 00:07'!
renderNavigationMenu: html
	html
		anchor
				callback: [ self returnHome ];
				with: 'Return Home' ! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 01:31'!
returnHome

	self answer! !

WAComponent subclass: #WebHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebHeader methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 01:48'!
renderContentOn: html

	html div
		id: 'header'; 
		with: [html heading: 'Home Automation System']! !

!WebHeader methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 14:57'!
style
	^ '
		#header {
			background: #CCCCCC;
		}'! !

WAComponent subclass: #WebLineChart
	instanceVariableNames: 'xdata ydata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebLineChart methodsFor: 'rendering' stamp: 'IsraSunhachawi 7/6/2013 15:29'!
renderContentOn: html
	html div
		script:
			(html
				highchart: [ :chart | 
					chart chart defaultSeriesType: 'line'.
					chart title text: 'Temperature'.
					chart xAxis categories: xdata.
					chart yAxis title text: 'Degrees Fahrenheit'.
					chart
						series:
							(Array
								with:
									(chart step
										name: 'Measured';
										data: ydata)) ])! !

!WebLineChart methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/5/2013 23:53'!
xData: xDataCollection yData: yDataCollection

super initialize.
xdata:= xDataCollection.
ydata:= yDataCollection.! !

Object subclass: #Alarm
	instanceVariableNames: 'threshold name device'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!Alarm methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 15:27'!
name

^name! !

!Alarm methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 15:00'!
setAlarm: aName device: aDevice threshold: aNumber

super initialize.
name := aName.
device := aDevice.
threshold := aNumber.! !

Object subclass: #Device
	instanceVariableNames: 'name ip sensors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!Device class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 22:03'!
newFromXml: xmlString
	|device aName anIp doc stream aSensor|
	device := Device new.
	stream := xmlString readStream.
	doc := XMLDOMParser parseDocumentFrom: stream.
	device name: (doc findElementNamed: 'name') contentString.
	device ip: (doc findElementNamed: 'ip') contentString.
	(doc allElementsNamed: 'sensor') do:
		[ :each | aSensor := DeviceSensorFactory createSensor: each contentString.
				aSensor ifNotNil: [device addSensor: aSensor] ].
	^device.! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 21:23'!
addSensor: aDeviceSensor
	"if the a sensor with the same type exists, it will be overwritten"
	sensors at: (aDeviceSensor type) put: aDeviceSensor.! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 22:06'!
asXml
	|stream|
	stream := WriteStream on: (String new: 1).
	stream nextPutAll: '<Device>'.
	stream nextPutAll: '<name>'.
	stream nextPutAll: name.
	stream nextPutAll: '</name>'.
	
	stream nextPutAll: '<ip>'.
	stream nextPutAll: ip.
	stream nextPutAll: '</ip>'.
	
	sensors keysDo: [ :key | 
		stream nextPutAll: '<sensor>'.
		stream nextPutAll: key.
		stream nextPutAll: '</sensor>'.].
	
	stream nextPutAll: '</Device>'.
	^stream contents.! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 22:13'!
getSensorByType: aTypeName
	^ sensors at: aTypeName.! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:53'!
initialize
	super initialize.
	sensors := Dictionary new.! !

!Device methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 00:40'!
ip
	^ip! !

!Device methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 00:39'!
ip: anIp
	ip := anIp! !

!Device methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 6/30/2013 23:29'!
name
	^name! !

!Device methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 6/30/2013 23:29'!
name: aName
	name := aName! !

!Device methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 00:59'!
postRawData: aRawData

data add: aRawData! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 21:16'!
sensors
	^sensors! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 21:17'!
sensors: aDict
	sensors := aDict! !

Object subclass: #DeviceManager
	instanceVariableNames: 'devices mutex'
	classVariableNames: 'uniqueInstance'
	poolDictionaries: ''
	category: 'Sensors'!

!DeviceManager class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 22:44'!
getDevice: aDeviceName
	^DeviceManager singleton getDevice: aDeviceName.! !

!DeviceManager class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:16'!
reset
	uniqueInstance := nil.! !

!DeviceManager class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 6/30/2013 23:27'!
singleton
	uniqueInstance ifNil: [uniqueInstance := self basicNew initialize].
       ^uniqueInstance! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 15:06'!
addAlarm: alarm
	alarms add: alarm! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 18:56'!
addDevice: device
	mutex critical: [ devices at: (device name) put: device ].
	! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 15:25'!
alarms
	^alarms! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 6/30/2013 23:29'!
devices
	^devices! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 10:30'!
getDevice: deviceName
	^devices at: deviceName.! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 18:56'!
initialize
	devices := Dictionary new.
	mutex := Mutex new.! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 18:57'!
removeDevice: deviceName
	mutex critical: [ devices removeKey: deviceName. ]! !

Object subclass: #DeviceSensor
	instanceVariableNames: 'type data dataUnit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:29'!
data
	^data
! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:30'!
initialize
	super initialize.
	self setType.
	self setUnit.
	data := ValueWithTimeStamp new.! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:37'!
printOn: aStream
	aStream nextPutAll: 'Sensor type: '; nextPutAll: type; cr.
	aStream nextPutAll: 'Sensor data: (in unit of: '; nextPutAll: dataUnit ; nextPut: $) ;cr.
	data printOn: aStream.! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:39'!
setType
	^self subclassResponsibility! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:30'!
setUnit
	^self subclassResponsibility! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:00'!
type
	^type
! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:24'!
updateData: anAmount at: aDateAndTime
	data at: aDateAndTime become: anAmount.
! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:25'!
updateDataNow: anAmount
	|aDateAndTime|
	aDateAndTime := DateAndTime now.
	self updateData: anAmount at: aDateAndTime.
! !

DeviceSensor subclass: #LightSensor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!LightSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:42'!
initialize
	super initialize.! !

!LightSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:43'!
setType
	type := #LightSensor! !

!LightSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:32'!
setUnit
	dataUnit := #Luminance! !

DeviceSensor subclass: #TemperatureSensor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!TemperatureSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:43'!
initialize
	super initialize.! !

!TemperatureSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:44'!
setType
	type := #TemperatureSensor! !

!TemperatureSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:33'!
setUnit
	dataUnit := #Fahrenheit! !

Object subclass: #DeviceSensorFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!DeviceSensorFactory class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 21:56'!
createSensor: aString
	aString = 'LightSensor' ifTrue: [^ LightSensor new].
	aString = 'TemperatureSensor' ifTrue: [^ TemperatureSensor new].
	^nil! !

Object subclass: #RawData
	instanceVariableNames: 'timestamp measurement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!RawData commentStamp: 'IsraSunhachawi 7/3/2013 01:33' prior: 0!
RawData is used to store measurements provided by sensors.

Instance Variables:
	deviceId	<ProtoObject | PseudoContext>
	timestamp	<ProtoObject | PseudoContext>
	measurement	<ProtoObject | PseudoContext>!

!RawData class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:40'!
createSampleDatabase
    Database := OrderedCollection new
        add: (self name: 'Bob Jones' emailAddress: 'bob@nowhere.com');
        add: (self name: 'Steve Smith' emailAddress: 'sm@somewhere.com');
        yourself! !

!RawData class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:39'!
device: deviceString timestamp: aTime measurement: aMeasurement
    ^ self new
          deviceId: deviceString;
          timestamp: aTime;
	    measurement: aMeasurement;
          yourself! !

!RawData methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:36'!
measurement

	^ measurement! !

!RawData methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:36'!
measurement: aNumber

	measurement := aNumber! !

!RawData methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 00:41'!
time: aTimeStamp measurement: aMeasurement

timestamp := aTimeStamp.
measurement := aMeasurement.! !

!RawData methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:35'!
timeStamp: aDate

	timestamp := aDate! !

!RawData methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:35'!
timestamp

	^ timestamp! !

Object subclass: #ValueWithTimeStamp
	instanceVariableNames: 'second value'
	classVariableNames: 'Epoch'
	poolDictionaries: ''
	category: 'Sensors'!

!ValueWithTimeStamp class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:42'!
initialize
	"self initialize"
	Epoch := DateAndTime fromSeconds: 0.! !

!ValueWithTimeStamp class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:42'!
new
	^ super new initialize
! !

!ValueWithTimeStamp class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:59'!
zero
	^ self new at: 0 become: 0
! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:13'!
at: aDate at: aTime become: anAmount 
	| index aDateAndTime aSecond|
	aDateAndTime := DateAndTime year: aDate year day: aDate day hour: aTime hour minute: aTime minute second: aTime second.
	self at: aDateAndTime become: anAmount.
! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:13'!
at: aDateAndTime become: anAmount 
	| index aSecond|
	aSecond := aDateAndTime asSeconds.
	self atSecond: aSecond become: anAmount.
! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:02'!
atSecond: aSecond become: anAmount 
	| index |
	second add: aSecond.
	index := second indexOfStartOfIntervalContaining: aSecond.
	value add: anAmount beforeIndex: index.! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:55'!
indexForAccessing: aSecond 
	"Create index of slot for aSecond, creating it if necessary."
	| index |
	(index := second indexOfStartOfIntervalContaining: aSecond) = 0 ifTrue: 
		[second add: aSecond.
		value addFirst: 0.
		^ 1].
	(second at: index) = aSecond ifTrue: [^index].
	second add: aSecond.
	value add: (value at: index) beforeIndex: index + 1.
	^ index + 1! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 10:52'!
initialize
	second := SortedSequence new.
	value := OrderedCollection new.
! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:48'!
printOn: aStream 
	second with: value do: 
		[: nextSecond : nextValue | 
		aStream 
			nextPut: $(;
			print: (DateAndTime fromSeconds: nextSecond);
			nextPutAll: ' -> ';
			print: nextValue;
			nextPut: $);
			cr].! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:54'!
second
	^second! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:52'!
timeStampAndValue
	|aDictionary aDateAndTime|
	aDictionary := Dictionary new.
	second with: value do: 
		[: nextSecond : nextValue | 
		aDateAndTime := DateAndTime fromSeconds: nextSecond.
		aDictionary at: aDateAndTime put: nextValue
		].
	^aDictionary! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 10:58'!
value
	^value! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:24'!
valueAt: aDateAndTime
	"Return value at aDateAndTime"
	| index aSecond|
	aSecond := aDateAndTime asSeconds.
	^ self valueAtSecond: aSecond
! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:23'!
valueAtSecond: aSecond 
	"Return value at aSecond"
	| index |
	index := second indexOfStartOfIntervalContaining: aSecond.
	index = 0 ifTrue: [self error: 'time is too early'].
	^value at: index! !
WebHome initialize!
ValueWithTimeStamp initialize!
